/**
 * synopsis: Generates an JWM menu xml segment based on an XDG menu structure
 * purpose: The purpose of this utility is to keep the JWM menu
 *          synchronized with the XDG configuration
 * usage: jwm-xdgmenu <xdgMenuFile>
 * author: Raul Suarez (rarsa at yahoo dot com)
 * Licence: GPL
 *
 * ammended: 01micko 6dec13 made height variable, accepts integer argument
 * 
 * Dependencies:
 *   gnome-menus 2.12
 *   glib
 */

#include <glib.h>
#include <glib/gprintf.h>
#include <gmenu-tree.h>


/*=============================================================================
 * Declarations
 */
static void show_help();
static void process_directory(GMenuTreeDirectory *directory, char *menheight);
static void process_entry(GMenuTreeEntry *entry);
static void process_separator(GMenuTreeSeparator *entry);
char *menheight;

/*=============================================================================
 * Main Function
 */
int
main (int argc, char **argv)
{

  if ((argc < 2)
      || (strcmp (argv[1], "-h") == 0)
      || (strcmp (argv[1], "-help") == 0)
      || (strcmp (argv[1], "--help") == 0))
  {
    show_help();
    return (1);
  }
  if ((argc == 3))
  {
	  if (strcmp (argv[2], "24") == 0)
	  { 
		  menheight = "24";
	  }
	  else if (strcmp (argv[2], "32") == 0)
	  { 
		  menheight = "32";
	  }
	  else if (strcmp (argv[2], "40") == 0)
	  { 
		  menheight = "40";
	  }
	  else if (strcmp (argv[2], "48") == 0)
	  { 
		  menheight = "48";
	  }
	  else
	  {
		  show_help();
		  return (1);
	  }
  }
  else
  {
	  menheight = "16";
  }
  
  GMenuTree *menuTree = gmenu_tree_lookup (argv[1],  GMENU_TREE_FLAGS_NONE );

  GMenuTreeDirectory *rootDirectory = gmenu_tree_get_root_directory(menuTree);

  process_directory(rootDirectory, menheight);

  gmenu_tree_item_unref (rootDirectory);

  return 0;

}

/*=============================================================================
 * Shows usage parameters
 */
void
show_help()
{
  printf ("Creates a JWM inlcude menu from an XDG menu structure.\n");
  printf ("Usage:\n");
  printf ("  jwm-xdgmenu xdgmenufile integer\n");
  printf ("  where \"integer\" (optional) is a legal value from the list below\n");
  printf ("\n");
  printf ("  xdgmenufile: Fully qualified path to the XDG menu file. This is a mandatory parameter\n");
  printf ("\n");
  printf ("For example:\n");
  printf ("  jwm-xdgmenu \"/etc/xdg/menus/applications.menu\"\n\n");
  printf ("  jwm-xdgmenu \"/etc/xdg/menus/applications.menu\" 24\n\n");
  printf ("  Allowed values are 24, 32, 40, 48. Default is 16\n\n");
}

/*=============================================================================
 * This function processes a directory entry and all it's child nodes
 */
void
process_directory(GMenuTreeDirectory *directory, char *menheight)
{
	int hasSeparator = 0;
	char *mheight = menheight;
	
  g_printf("<Menu label=\"%s\" icon=\"%s\" height=\"%s\">\n",
            gmenu_tree_directory_get_name(directory),
            gmenu_tree_directory_get_icon(directory),
            mheight);

  GMenuTreeItemType entryType;

  GSList *entryList = gmenu_tree_directory_get_contents (directory);

  GSList *l;

  for (l = entryList; l; l = l->next)
  {

    GMenuTreeItem *item = l->data;

    entryType = gmenu_tree_item_get_type (GMENU_TREE_ITEM(item));

	switch (entryType)
	{
		case GMENU_TREE_ITEM_DIRECTORY:
		  if (hasSeparator)
		  {
				process_separator(GMENU_TREE_SEPARATOR(item));
				hasSeparator = 0;
		  }
			process_directory(GMENU_TREE_DIRECTORY(item),menheight);
			break;
		case GMENU_TREE_ITEM_ENTRY:
		  if (hasSeparator)
		  {
				process_separator(GMENU_TREE_SEPARATOR(item));
				hasSeparator = 0;
		  }
			process_entry(GMENU_TREE_ENTRY(item));
			break;
		case GMENU_TREE_ITEM_SEPARATOR:
			hasSeparator = 1;
			break;
	}

    gmenu_tree_item_unref (item);

  }
  g_printf("</Menu>\n");
  g_slist_free (entryList);
}

/*=============================================================================
 * This function adds an application entry
 */
void
process_entry(GMenuTreeEntry *entry)
{

  g_printf("<Program label=\"%s\" icon=\"%s\">%s</Program>\n",
            gmenu_tree_entry_get_name(entry),
            gmenu_tree_entry_get_icon (entry),
            gmenu_tree_entry_get_exec (entry));
}

/*=============================================================================
 * This function adds a separator
 */
void
process_separator(GMenuTreeSeparator *entry)
{

  g_printf(" <Separator/> \n");
}

/*=============================================================================
 */
